
#define ERROR -1
#define VALID 0

#define HELP_FLAG "-?"
#define FILENAME_FLAG "-file"
#define SAMPLERATE_FLAG "-sr"
#define SINE_FREQ_FLAG "-frq"
#define FILESIZE_FLAG "-kb"
#define NUM_CHANNELS_FLAG "-ch"
#define SAMPLESIZE_FLAG "-res"
#define DAMPING_FLAG "-db"
#define MODULATE_FREQ_FLAG "-mfrq"
#define MODULATE_RANGE_FLAG "-mran"

#define RIFF_STRING "RIFF"
#define WAVE_STRING "WAVEfmt "
#define DATA_STRING "data"

#define SHOW_STATUS

#define TWO_PI 6.283185307179586476925286766559
#define DEFAULT_FILENAME "output.wav"


#define MIN_MOD_FREQ_HZ 0.0 // 0 Hz
#define MAX_MOD_FREQ_HZ 1000.0 // 1000 Hz

#define DEFAULT_SINE_FREQ_HZ 440 // 440Hz
#define DEFAULT_SAMPLERATE_HZ 48000 // 48kHz
#define DEFAULT_MOD_FREQ_HZ 0 // 0 Hz
#define DEFAULT_DAMPING_DB -3.0 // -3 dB
#define DEFAULT_MOD_RANGE_PERCENT 100.0 // Equals PI

#define	SIZEOF_BYTE 8

#define NUM_BYTES_PER_KB 1024
#define NUM_BYTES_FMT_HEADER 16
#define NUM_BYTES_WAVE_FILE_HEADER 44
#define PCM_FORMAT 0x001

#define MIN_MOD_RANGE_PERCENT 0.0
#define MAX_MOD_RANGE_PERCENT 100.0
	
#define MIN_SAMPLERATE_HZ 1000 // 1 kHz
#define MAX_SAMPLERATE_HZ 96000 // 96 kHz
		
#define MIN_SINE_FREQ_HZ 1.0 // Hz
#define MAX_SINE_FREQ_HZ (MAX_SAMPLERATE_HZ / 2.0) // Nyquist

#define MIN_FILESIZE_IN_KB 1 // 1kB
#define MAX_FILESIZE_IN_KB 1000000 // 1 GB
#define DEFAULT_FILESIZE_IN_KB 5000 // 5MB

#define MIN_NUM_CHANNELS 1
#define MAX_NUM_CHANNELS 6
#define DEFAULT_NUM_CHANNELS 2 // Stereo default
	
#define MIN_NUM_BYTES_PER_SAMPLE 1
#define MAX_NUM_BYTES_PER_SAMPLE 4
#define DEFAULT_NUM_BYTES_PER_SAMPLE 2 // 16 bit default

#define MIN_DAMPING_DB -45
#define MAX_DAMPING_DB 0.0

/**
 * Parameters describing a file object
 */
struct FileObject {
	const char* name;
	int numBytes;
};

/**
 * PCM parameters
 */
struct PCMParameters {
	int sampleRateHz;
	int numBytesPerSample;
	int numChannels;
};

/**
 * The parameters specifying a LFO
 * (Low frequency oscillator)
 */
struct LFOParameters {
	double freqHz;
	double amplitude;
	double phase;
};

/**
 * Returns if
 * value = 20 * log10(x)
 */
double dBToLinear(double value);

/**
 * Returns modulator range
 * @param percent
 * Value from 0 to 100% where 100% equals 2*PI
 */
double getModRange(double percent);

/**
 * Parses the command line arguments and sets
 * the local values according to the user input
 */
int parseArguments(int numArgs, const char* arg[]);

/**
 * Writes the bytes of valut to the output stream
 * @param numBytes
 * The number of bytes which shall be written
 * @param value
 * The value that contains the bytes that shall be written
 */
void writeLittleEndian(FILE* file, unsigned int value, int numBytes);

/**
 * Writes the char array to the file output stream.
 */
void writeLittleEndian(FILE* file, const char* str);

/**
 * Writes the wave header to the file output stream.
 * This function relies on the PCMParameters of the pcm struct
 * to work correctly. Hence, those values have to be set before
 * calling this function.
 * 
 * @return
 * The number of PCM frames that have
 * been defined to be written.
 */
int writeWAVheader(FILE* file);

/**
 * Writes the wave file including the wave header by calling the
 * writeWAVHeader function.
 * The wave file contains the sound of a sine oscillator (LFO)
 * which has been configured by the user arguments.
 * 
 * @return
 * VALID wave file was sucessfully written
 * ERROR error opening the wave file
 */
int writeWAVFile();

/**
 * Drops an argument help message 
 * on the command line
 */
void dropHelpMessage();

/**
 * MAIN
 */
int main(int numArgs, const char* arg[]);

